Attribute VB_Name = "AccesoINI"
Option Explicit
' ################################################################################################################
' RFM 2006
' ################################################################################################################
'
' Acceso INI (actualizacin del componente INI_Files), esta versin incluye mejoras y utiliza API's para
' acceder/guardar los datos del archivo INI
'
' Lista de funciones:
' -------------------
'
'   integer = KeyCount(archivoINI) <----------------------------------------- devuelve la cantidad de Keys (llaves)
'   string = KeyList(indice,archivoINI) <------------------------------------ devuelve texto key indicada
'   string = INIRead("Key/Variable",archivoINI) <---------------------------- devuelve valor variable indicada
'   INIAdd "Key/Variable", "Valor", archivoINI <----------------------------- agrega/modifica valor
'   INISort archivoINI <----------------------------------------------------- "ordena" el archivo ini
'
' ################################################################################################################
' Declaraciones y definiciones
' ################################################################################################################
Private Declare Function GetPrivateProfileString Lib "kernel32" Alias "GetPrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpDefault As String, ByVal lpReturnedString As String, ByVal nSize As Long, ByVal lpFileName As String) As Long
Private Declare Function WritePrivateProfileString Lib "kernel32" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As Any, ByVal lpString As Any, ByVal lpFileName As String) As Long
Enum Tipo_Respuesta
    ConMensajes = 0
    SinMensajes = 1
End Enum
' Definiendo variables internas de comportamiento
Dim T_ERRORES As Tipo_Respuesta
Dim S_ARCHIVO_INI As String * 255 ' Ruta del archivo
' Definimos el carcter separador para mdulo
Const S_CARACTER As String * 1 = "/"
' ################################################################################################################
' Funciones y rutinas privadas
' ################################################################################################################
Private Function APIToVBStr(Cadena$)
  Dim Pos%
  APIToVBStr = Cadena$
  Pos = InStr(APIToVBStr, Chr(0))
  If Pos > 0 Then APIToVBStr = Left$(APIToVBStr, Pos - 1)
End Function
Private Function PasaPalabra(ByVal Cadena As String, ByVal CaracterSeparador As String, ByVal Indice As Integer) As String
Dim a1 As Integer, a2 As Integer, tmp As String, Contador As Integer

    If Right(RTrim(Cadena), 1) <> CaracterSeparador Then
        Cadena = Cadena + CaracterSeparador
    End If
    
    Do
        Contador = Contador + 1
        a1 = InStr(a2 + 1, Cadena, CaracterSeparador)
        If a1 < 1 Then tmp = "": Exit Do
        tmp = Mid(Cadena, a2 + 1, (a1 - 1) - a2)
        If Contador = Indice Then Exit Do
        a2 = a1
    Loop
    
    'Pasando la cabra
    PasaPalabra = tmp
End Function

' ################################################################################################################
' Funciones y rutinas pblicas
' ################################################################################################################
' Devuelve la cantidad de llaves "[]" existentes en el archivo INI
Public Function KeyCount(ByVal ArchivoINI As String) As Integer
Dim ll As Integer, sLinea As String, nContador As Integer
Dim sArchivoINI As String
    ' Definimos el archivo con el que trabajar
     sArchivoINI = ArchivoINI
    ll = FreeFile
    On Error GoTo NoSeAbre
    Open sArchivoINI For Input As #ll
        Do While Not EOF(ll)
            Line Input #ll, sLinea
            sLinea = Trim(sLinea)
            If Mid(sLinea, 1, 1) = "[" Then
                If Mid(sLinea, Len(sLinea)) = "]" Then nContador = nContador + 1
            End If
        Loop
    Close #ll
    GoTo ok
NoSeAbre:
    If T_ERRORES = SinMensajes Then
        Resume ok
    Else
        MsgBox "No se encontr el archivo indicado o existieron problemas al acceder.", vbCritical
        Resume ok
    End If
ok:
KeyCount = nContador
On Error GoTo 0
End Function

' Devuelve el nombre de la llave "[]" indicada por ndice (ubicacin)
Public Function KeyList(ByVal nValor As Integer, ByVal ArchivoINI As String) As String
Dim ll As Integer, sLinea As String, Contador As Integer
Dim sArchivoINI As String
    ' Definimos el archivo con el que trabajar
    sArchivoINI = ArchivoINI
    ll = FreeFile
    On Error GoTo NoSeAbre
    Open sArchivoINI For Input As #ll
        Do While Not EOF(ll)
            Line Input #ll, sLinea
            sLinea = Trim(sLinea)
            If Mid(sLinea, 1, 1) = "[" Then
                If Mid(sLinea, Len(sLinea)) = "]" Then Contador = Contador + 1
                If Contador - 1 = nValor Then
                    sLinea = Mid(sLinea, 2)
                    KeyList = Mid(sLinea, 1, Len(sLinea) - 1)
                    Exit Do
                End If
            End If
        Loop
    Close #ll
    GoTo ok
NoSeAbre:
    If T_ERRORES = SinMensajes Then
        Resume ok
    Else
        MsgBox "No se encontr el archivo indicado o existieron problemas al acceder.", vbCritical
        Resume ok
    End If
ok:
On Error GoTo 0
End Function

' Devuelve el valor indicado por 'Variable' usando API's de Windows
Public Function INIRead(ByVal Variable As String, ByVal ArchivoINI As String) As String
Dim v_A As String, v_V As String, Respuesta As String * 255
Dim sArchivoINI As String
    ' Definimos el archivo con el que trabajar
    sArchivoINI = ArchivoINI
    ' Cargamos variables pasadas
    v_A = PasaPalabra(Variable, S_CARACTER, 1)
    v_V = PasaPalabra(Variable, S_CARACTER, 2)
    ' Realizamos la consulta al archivo INI
    GetPrivateProfileString v_A, v_V, "", Respuesta, 255, sArchivoINI
    INIRead = Trim(APIToVBStr(Respuesta))
End Function

' Agrega/modifica la variable indicada en el archivo INI
Public Sub INIAdd(ByVal Variable As String, ByVal Valor As String, ByVal ArchivoINI As String)
Dim v_A As String, v_V As String, Respuesta As String * 255
Dim sArchivoINI As String
    ' Definimos el archivo con el que trabajar
    sArchivoINI = ArchivoINI
    ' Cargamos variables pasadas
    v_A = PasaPalabra(Variable, S_CARACTER, 1)
    v_V = PasaPalabra(Variable, S_CARACTER, 2)
    WritePrivateProfileString v_A, v_V, Valor, sArchivoINI
End Sub

' Ordena el archivo
Public Sub INISort(ByVal ArchivoINI As String)
Dim l1 As Integer, l2 As Integer
Dim Linea As String
Dim sArchivoINI As String
    ' Definimos el archivo con el que trabajar
    sArchivoINI = ArchivoINI
    l1 = FreeFile
    On Error GoTo CGda
    Open sArchivoINI For Input As #l1
        l2 = FreeFile
        Open App.Path + "\Tmp_INI.tmp" For Output As #l2
        Do While Not EOF(l1)
            Line Input #l1, Linea
            Linea = Trim(Linea)
            If Linea <> "" Then
                If Mid(Linea, 1, 1) = "[" Then
                    Print #l2, ""
                End If
                Print #l2, Linea
            End If
        Loop
        Close #l2
    Close #l1
    Kill sArchivoINI
    Name App.Path + "\Tmp_INI.tmp" As sArchivoINI
    GoTo salto
CGda:
    Resume salto
salto:
On Error GoTo 0
End Sub
